# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .terminal_action import TerminalActionParams


class DismissTerminalActionResponseParams(typing_extensions.TypedDict):
    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Information on errors encountered during the request.
    """

    action: typing_extensions.NotRequired[TerminalActionParams]
    """
    Current state of the action to be dismissed.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
from .invoice_filter import InvoiceFilter
import pydantic
import typing
from .invoice_sort import InvoiceSort
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class InvoiceQuery(UncheckedBaseModel):
    """
    Describes query criteria for searching invoices.
    """

    filter: InvoiceFilter = pydantic.Field()
    """
    Query filters to apply in searching invoices. 
    For more information, see [Search for invoices](https://developer.squareup.com/docs/invoices-api/retrieve-list-search-invoices#search-invoices).
    """

    sort: typing.Optional[InvoiceSort] = pydantic.Field(default=None)
    """
    Describes the sort order for the search result.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class DeviceComponentDetailsEthernetDetailsParams(typing_extensions.TypedDict):
    active: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    A boolean to represent whether the Ethernet interface is currently active.
    """

    ip_address_v4: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The string representation of the device’s IPv4 address.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .subscription import SubscriptionParams


class SearchSubscriptionsResponseParams(typing_extensions.TypedDict):
    """
    Defines output parameters in a response from the
    [SearchSubscriptions](api-endpoint:Subscriptions-SearchSubscriptions) endpoint.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Errors encountered during the request.
    """

    subscriptions: typing_extensions.NotRequired[typing.Sequence[SubscriptionParams]]
    """
    The subscriptions matching the specified query expressions.
    """

    cursor: typing_extensions.NotRequired[str]
    """
    When the total number of resulting subscription exceeds the limit of a paged response, 
    the response includes a cursor for you to use in a subsequent request to fetch the next set of results.
    If the cursor is unset, the response contains the last page of the results.
    
    For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .order import OrderParams


class CreateOrderRequestParams(typing_extensions.TypedDict):
    order: typing_extensions.NotRequired[OrderParams]
    """
    The order to create. If this field is set, the only other top-level field that can be
    set is the `idempotency_key`.
    """

    idempotency_key: typing_extensions.NotRequired[str]
    """
    A value you specify that uniquely identifies this
    order among orders you have created.
    
    If you are unsure whether a particular order was created successfully,
    you can try it again with the same idempotency key without
    worrying about creating duplicate orders.
    
    For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).
    """



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from .raw_client import RawCustomAttributeDefinitionsClient
from ...types.visibility_filter import VisibilityFilter
from ...core.request_options import RequestOptions
from ...core.pagination import SyncPager
from ...types.custom_attribute_definition import CustomAttributeDefinition
from ...types.list_location_custom_attribute_definitions_response import ListLocationCustomAttributeDefinitionsResponse
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...requests.custom_attribute_definition import CustomAttributeDefinitionParams
from ...types.create_location_custom_attribute_definition_response import (
    CreateLocationCustomAttributeDefinitionResponse,
)
from ...types.retrieve_location_custom_attribute_definition_response import (
    RetrieveLocationCustomAttributeDefinitionResponse,
)
from ...types.update_location_custom_attribute_definition_response import (
    UpdateLocationCustomAttributeDefinitionResponse,
)
from ...types.delete_location_custom_attribute_definition_response import (
    DeleteLocationCustomAttributeDefinitionResponse,
)
from ...core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawCustomAttributeDefinitionsClient
from ...core.pagination import AsyncPager

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CustomAttributeDefinitionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCustomAttributeDefinitionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawCustomAttributeDefinitionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCustomAttributeDefinitionsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        visibility_filter: typing.Optional[VisibilityFilter] = None,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[CustomAttributeDefinition]:
        """
        Lists the location-related [custom attribute definitions](entity:CustomAttributeDefinition) that belong to a Square seller account.
        When all response pages are retrieved, the results include all custom attribute definitions
        that are visible to the requesting application, including those that are created by other
        applications and set to `VISIBILITY_READ_ONLY` or `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        visibility_filter : typing.Optional[VisibilityFilter]
            Filters the `CustomAttributeDefinition` results by their `visibility` values.

        limit : typing.Optional[int]
            The maximum number of results to return in a single paged response. This limit is advisory.
            The response might contain more or fewer results. The minimum value is 1 and the maximum value is 100.
            The default value is 20. For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        cursor : typing.Optional[str]
            The cursor returned in the paged response from the previous call to this endpoint.
            Provide this cursor to retrieve the next page of results for your original request.
            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[CustomAttributeDefinition]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        response = client.locations.custom_attribute_definitions.list()
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        _response = self._raw_client._client_wrapper.httpx_client.request(
            "v2/locations/custom-attribute-definitions",
            method="GET",
            params={
                "visibility_filter": visibility_filter,
                "limit": limit,
                "cursor": cursor,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListLocationCustomAttributeDefinitionsResponse,
                    construct_type(
                        type_=ListLocationCustomAttributeDefinitionsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    visibility_filter=visibility_filter,
                    limit=limit,
                    cursor=_parsed_next,
                    request_options=request_options,
                )
                _items = _parsed_response.custom_attribute_definitions
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        custom_attribute_definition: CustomAttributeDefinitionParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateLocationCustomAttributeDefinitionResponse:
        """
        Creates a location-related [custom attribute definition](entity:CustomAttributeDefinition) for a Square seller account.
        Use this endpoint to define a custom attribute that can be associated with locations.
        A custom attribute definition specifies the `key`, `visibility`, `schema`, and other properties
        for a custom attribute. After the definition is created, you can call
        [UpsertLocationCustomAttribute](api-endpoint:LocationCustomAttributes-UpsertLocationCustomAttribute) or
        [BulkUpsertLocationCustomAttributes](api-endpoint:LocationCustomAttributes-BulkUpsertLocationCustomAttributes)
        to set the custom attribute for locations.

        Parameters
        ----------
        custom_attribute_definition : CustomAttributeDefinitionParams
            The custom attribute definition to create. Note the following:
            - With the exception of the `Selection` data type, the `schema` is specified as a simple URL to the JSON schema
            definition hosted on the Square CDN. For more information, including supported values and constraints, see
            [Supported data types](https://developer.squareup.com/docs/devtools/customattributes/overview#supported-data-types).
            - `name` is required unless `visibility` is set to `VISIBILITY_HIDDEN`.

        idempotency_key : typing.Optional[str]
            A unique identifier for this request, used to ensure idempotency. For more information,
            see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateLocationCustomAttributeDefinitionResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.locations.custom_attribute_definitions.create(
            custom_attribute_definition={
                "key": "bestseller",
                "schema": {
                    "ref": "https://developer-production-s.squarecdn.com/schemas/v1/common.json#squareup.common.String"
                },
                "name": "Bestseller",
                "description": "Bestselling item at location",
                "visibility": "VISIBILITY_READ_WRITE_VALUES",
            },
        )
        """
        response = self._raw_client.create(
            custom_attribute_definition=custom_attribute_definition,
            idempotency_key=idempotency_key,
            request_options=request_options,
        )
        return response.data

    def get(
        self, key: str, *, version: typing.Optional[int] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> RetrieveLocationCustomAttributeDefinitionResponse:
        """
        Retrieves a location-related [custom attribute definition](entity:CustomAttributeDefinition) from a Square seller account.
        To retrieve a custom attribute definition created by another application, the `visibility`
        setting must be `VISIBILITY_READ_ONLY` or `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to retrieve. If the requesting application
            is not the definition owner, you must use the qualified key.

        version : typing.Optional[int]
            The current version of the custom attribute definition, which is used for strongly consistent
            reads to guarantee that you receive the most up-to-date data. When included in the request,
            Square returns the specified version or a higher version if one exists. If the specified version
            is higher than the current version, Square returns a `BAD_REQUEST` error.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RetrieveLocationCustomAttributeDefinitionResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.locations.custom_attribute_definitions.get(
            key="key",
        )
        """
        response = self._raw_client.get(key, version=version, request_options=request_options)
        return response.data

    def update(
        self,
        key: str,
        *,
        custom_attribute_definition: CustomAttributeDefinitionParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateLocationCustomAttributeDefinitionResponse:
        """
        Updates a location-related [custom attribute definition](entity:CustomAttributeDefinition) for a Square seller account.
        Use this endpoint to update the following fields: `name`, `description`, `visibility`, or the
        `schema` for a `Selection` data type.
        Only the definition owner can update a custom attribute definition.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to update.

        custom_attribute_definition : CustomAttributeDefinitionParams
            The custom attribute definition that contains the fields to update. This endpoint
            supports sparse updates, so only new or changed fields need to be included in the request.
            Only the following fields can be updated:
            - `name`
            - `description`
            - `visibility`
            - `schema` for a `Selection` data type. Only changes to the named options or the maximum number of allowed
            selections are supported.

            For more information, see
            [Update a location custom attribute definition](https://developer.squareup.com/docs/location-custom-attributes-api/custom-attribute-definitions#update-custom-attribute-definition).
            To enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
            control, specify the current version of the custom attribute definition.
            If this is not important for your application, `version` can be set to -1.

        idempotency_key : typing.Optional[str]
            A unique identifier for this request, used to ensure idempotency. For more information,
            see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateLocationCustomAttributeDefinitionResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.locations.custom_attribute_definitions.update(
            key="key",
            custom_attribute_definition={
                "description": "Update the description as desired.",
                "visibility": "VISIBILITY_READ_ONLY",
            },
        )
        """
        response = self._raw_client.update(
            key,
            custom_attribute_definition=custom_attribute_definition,
            idempotency_key=idempotency_key,
            request_options=request_options,
        )
        return response.data

    def delete(
        self, key: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteLocationCustomAttributeDefinitionResponse:
        """
        Deletes a location-related [custom attribute definition](entity:CustomAttributeDefinition) from a Square seller account.
        Deleting a custom attribute definition also deletes the corresponding custom attribute from
        all locations.
        Only the definition owner can delete a custom attribute definition.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteLocationCustomAttributeDefinitionResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.locations.custom_attribute_definitions.delete(
            key="key",
        )
        """
        response = self._raw_client.delete(key, request_options=request_options)
        return response.data


class AsyncCustomAttributeDefinitionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCustomAttributeDefinitionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawCustomAttributeDefinitionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCustomAttributeDefinitionsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        visibility_filter: typing.Optional[VisibilityFilter] = None,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[CustomAttributeDefinition]:
        """
        Lists the location-related [custom attribute definitions](entity:CustomAttributeDefinition) that belong to a Square seller account.
        When all response pages are retrieved, the results include all custom attribute definitions
        that are visible to the requesting application, including those that are created by other
        applications and set to `VISIBILITY_READ_ONLY` or `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        visibility_filter : typing.Optional[VisibilityFilter]
            Filters the `CustomAttributeDefinition` results by their `visibility` values.

        limit : typing.Optional[int]
            The maximum number of results to return in a single paged response. This limit is advisory.
            The response might contain more or fewer results. The minimum value is 1 and the maximum value is 100.
            The default value is 20. For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        cursor : typing.Optional[str]
            The cursor returned in the paged response from the previous call to this endpoint.
            Provide this cursor to retrieve the next page of results for your original request.
            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[CustomAttributeDefinition]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.locations.custom_attribute_definitions.list()
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            "v2/locations/custom-attribute-definitions",
            method="GET",
            params={
                "visibility_filter": visibility_filter,
                "limit": limit,
                "cursor": cursor,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListLocationCustomAttributeDefinitionsResponse,
                    construct_type(
                        type_=ListLocationCustomAttributeDefinitionsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    visibility_filter=visibility_filter,
                    limit=limit,
                    cursor=_parsed_next,
                    request_options=request_options,
                )
                _items = _parsed_response.custom_attribute_definitions
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        custom_attribute_definition: CustomAttributeDefinitionParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateLocationCustomAttributeDefinitionResponse:
        """
        Creates a location-related [custom attribute definition](entity:CustomAttributeDefinition) for a Square seller account.
        Use this endpoint to define a custom attribute that can be associated with locations.
        A custom attribute definition specifies the `key`, `visibility`, `schema`, and other properties
        for a custom attribute. After the definition is created, you can call
        [UpsertLocationCustomAttribute](api-endpoint:LocationCustomAttributes-UpsertLocationCustomAttribute) or
        [BulkUpsertLocationCustomAttributes](api-endpoint:LocationCustomAttributes-BulkUpsertLocationCustomAttributes)
        to set the custom attribute for locations.

        Parameters
        ----------
        custom_attribute_definition : CustomAttributeDefinitionParams
            The custom attribute definition to create. Note the following:
            - With the exception of the `Selection` data type, the `schema` is specified as a simple URL to the JSON schema
            definition hosted on the Square CDN. For more information, including supported values and constraints, see
            [Supported data types](https://developer.squareup.com/docs/devtools/customattributes/overview#supported-data-types).
            - `name` is required unless `visibility` is set to `VISIBILITY_HIDDEN`.

        idempotency_key : typing.Optional[str]
            A unique identifier for this request, used to ensure idempotency. For more information,
            see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateLocationCustomAttributeDefinitionResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.locations.custom_attribute_definitions.create(
                custom_attribute_definition={
                    "key": "bestseller",
                    "schema": {
                        "ref": "https://developer-production-s.squarecdn.com/schemas/v1/common.json#squareup.common.String"
                    },
                    "name": "Bestseller",
                    "description": "Bestselling item at location",
                    "visibility": "VISIBILITY_READ_WRITE_VALUES",
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.create(
            custom_attribute_definition=custom_attribute_definition,
            idempotency_key=idempotency_key,
            request_options=request_options,
        )
        return response.data

    async def get(
        self, key: str, *, version: typing.Optional[int] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> RetrieveLocationCustomAttributeDefinitionResponse:
        """
        Retrieves a location-related [custom attribute definition](entity:CustomAttributeDefinition) from a Square seller account.
        To retrieve a custom attribute definition created by another application, the `visibility`
        setting must be `VISIBILITY_READ_ONLY` or `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to retrieve. If the requesting application
            is not the definition owner, you must use the qualified key.

        version : typing.Optional[int]
            The current version of the custom attribute definition, which is used for strongly consistent
            reads to guarantee that you receive the most up-to-date data. When included in the request,
            Square returns the specified version or a higher version if one exists. If the specified version
            is higher than the current version, Square returns a `BAD_REQUEST` error.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RetrieveLocationCustomAttributeDefinitionResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.locations.custom_attribute_definitions.get(
                key="key",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(key, version=version, request_options=request_options)
        return response.data

    async def update(
        self,
        key: str,
        *,
        custom_attribute_definition: CustomAttributeDefinitionParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateLocationCustomAttributeDefinitionResponse:
        """
        Updates a location-related [custom attribute definition](entity:CustomAttributeDefinition) for a Square seller account.
        Use this endpoint to update the following fields: `name`, `description`, `visibility`, or the
        `schema` for a `Selection` data type.
        Only the definition owner can update a custom attribute definition.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to update.

        custom_attribute_definition : CustomAttributeDefinitionParams
            The custom attribute definition that contains the fields to update. This endpoint
            supports sparse updates, so only new or changed fields need to be included in the request.
            Only the following fields can be updated:
            - `name`
            - `description`
            - `visibility`
            - `schema` for a `Selection` data type. Only changes to the named options or the maximum number of allowed
            selections are supported.

            For more information, see
            [Update a location custom attribute definition](https://developer.squareup.com/docs/location-custom-attributes-api/custom-attribute-definitions#update-custom-attribute-definition).
            To enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
            control, specify the current version of the custom attribute definition.
            If this is not important for your application, `version` can be set to -1.

        idempotency_key : typing.Optional[str]
            A unique identifier for this request, used to ensure idempotency. For more information,
            see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateLocationCustomAttributeDefinitionResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.locations.custom_attribute_definitions.update(
                key="key",
                custom_attribute_definition={
                    "description": "Update the description as desired.",
                    "visibility": "VISIBILITY_READ_ONLY",
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.update(
            key,
            custom_attribute_definition=custom_attribute_definition,
            idempotency_key=idempotency_key,
            request_options=request_options,
        )
        return response.data

    async def delete(
        self, key: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteLocationCustomAttributeDefinitionResponse:
        """
        Deletes a location-related [custom attribute definition](entity:CustomAttributeDefinition) from a Square seller account.
        Deleting a custom attribute definition also deletes the corresponding custom attribute from
        all locations.
        Only the definition owner can delete a custom attribute definition.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteLocationCustomAttributeDefinitionResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.locations.custom_attribute_definitions.delete(
                key="key",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.delete(key, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class InvoiceAttachment(UncheckedBaseModel):
    """
    Represents a file attached to an [invoice](entity:Invoice).
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Square-assigned ID of the attachment.
    """

    filename: typing.Optional[str] = pydantic.Field(default=None)
    """
    The file name of the attachment, which is displayed on the invoice.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    The description of the attachment, which is displayed on the invoice.
    This field maps to the seller-defined **Message** field.
    """

    filesize: typing.Optional[int] = pydantic.Field(default=None)
    """
    The file size of the attachment in bytes.
    """

    hash: typing.Optional[str] = pydantic.Field(default=None)
    """
    The MD5 hash that was generated from the file contents.
    """

    mime_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The mime type of the attachment.
    The following mime types are supported: 
    image/gif, image/jpeg, image/png, image/tiff, image/bmp, application/pdf.
    """

    uploaded_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when the attachment was uploaded, in RFC 3339 format.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .invoice_attachment import InvoiceAttachmentParams
import typing
from .error import ErrorParams


class CreateInvoiceAttachmentResponseParams(typing_extensions.TypedDict):
    """
    Represents a [CreateInvoiceAttachment](api-endpoint:Invoices-CreateInvoiceAttachment) response.
    """

    attachment: typing_extensions.NotRequired[InvoiceAttachmentParams]
    """
    Metadata about the attachment that was added to the invoice.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Information about errors encountered during the request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .order_line_item_pricing_blocklists_blocked_discount import OrderLineItemPricingBlocklistsBlockedDiscountParams
from .order_line_item_pricing_blocklists_blocked_tax import OrderLineItemPricingBlocklistsBlockedTaxParams


class OrderLineItemPricingBlocklistsParams(typing_extensions.TypedDict):
    """
    Describes pricing adjustments that are blocked from automatic
    application to a line item. For more information, see
    [Apply Taxes and Discounts](https://developer.squareup.com/docs/orders-api/apply-taxes-and-discounts).
    """

    blocked_discounts: typing_extensions.NotRequired[
        typing.Optional[typing.Sequence[OrderLineItemPricingBlocklistsBlockedDiscountParams]]
    ]
    """
    A list of discounts blocked from applying to the line item.
    Discounts can be blocked by the `discount_uid` (for ad hoc discounts) or
    the `discount_catalog_object_id` (for catalog discounts).
    """

    blocked_taxes: typing_extensions.NotRequired[
        typing.Optional[typing.Sequence[OrderLineItemPricingBlocklistsBlockedTaxParams]]
    ]
    """
    A list of taxes blocked from applying to the line item.
    Taxes can be blocked by the `tax_uid` (for ad hoc taxes) or
    the `tax_catalog_object_id` (for catalog taxes).
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .ach_details import AchDetails
from .error import Error
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class BankAccountPaymentDetails(UncheckedBaseModel):
    """
    Additional details about BANK_ACCOUNT type payments.
    """

    bank_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the bank associated with the bank account.
    """

    transfer_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The type of the bank transfer. The type can be `ACH` or `UNKNOWN`.
    """

    account_ownership_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ownership type of the bank account performing the transfer.
    The type can be `INDIVIDUAL`, `COMPANY`, or `ACCOUNT_TYPE_UNKNOWN`.
    """

    fingerprint: typing.Optional[str] = pydantic.Field(default=None)
    """
    Uniquely identifies the bank account for this seller and can be used
    to determine if payments are from the same bank account.
    """

    country: typing.Optional[str] = pydantic.Field(default=None)
    """
    The two-letter ISO code representing the country the bank account is located in.
    """

    statement_description: typing.Optional[str] = pydantic.Field(default=None)
    """
    The statement description as sent to the bank.
    """

    ach_details: typing.Optional[AchDetails] = pydantic.Field(default=None)
    """
    ACH-specific information about the transfer. The information is only populated
    if the `transfer_type` is `ACH`.
    """

    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    Information about errors encountered during the request.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .terminal_checkout_query_filter import TerminalCheckoutQueryFilterParams
from .terminal_checkout_query_sort import TerminalCheckoutQuerySortParams


class TerminalCheckoutQueryParams(typing_extensions.TypedDict):
    filter: typing_extensions.NotRequired[TerminalCheckoutQueryFilterParams]
    """
    Options for filtering returned `TerminalCheckout` objects.
    """

    sort: typing_extensions.NotRequired[TerminalCheckoutQuerySortParams]
    """
    Option for sorting returned `TerminalCheckout` objects.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.client_wrapper import SyncClientWrapper
import typing
from ..core.request_options import RequestOptions
from ..core.http_response import HttpResponse
from ..types.get_bank_account_by_v1id_response import GetBankAccountByV1IdResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..types.get_bank_account_response import GetBankAccountResponse
from ..core.client_wrapper import AsyncClientWrapper
from ..core.http_response import AsyncHttpResponse


class RawBankAccountsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_by_v1id(
        self, v1bank_account_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetBankAccountByV1IdResponse]:
        """
        Returns details of a [BankAccount](entity:BankAccount) identified by V1 bank account ID.

        Parameters
        ----------
        v1bank_account_id : str
            Connect V1 ID of the desired `BankAccount`. For more information, see
            [Retrieve a bank account by using an ID issued by V1 Bank Accounts API](https://developer.squareup.com/docs/bank-accounts-api#retrieve-a-bank-account-by-using-an-id-issued-by-v1-bank-accounts-api).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetBankAccountByV1IdResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/bank-accounts/by-v1-id/{jsonable_encoder(v1bank_account_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetBankAccountByV1IdResponse,
                    construct_type(
                        type_=GetBankAccountByV1IdResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self, bank_account_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetBankAccountResponse]:
        """
        Returns details of a [BankAccount](entity:BankAccount)
        linked to a Square account.

        Parameters
        ----------
        bank_account_id : str
            Square-issued ID of the desired `BankAccount`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetBankAccountResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/bank-accounts/{jsonable_encoder(bank_account_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetBankAccountResponse,
                    construct_type(
                        type_=GetBankAccountResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawBankAccountsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_by_v1id(
        self, v1bank_account_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetBankAccountByV1IdResponse]:
        """
        Returns details of a [BankAccount](entity:BankAccount) identified by V1 bank account ID.

        Parameters
        ----------
        v1bank_account_id : str
            Connect V1 ID of the desired `BankAccount`. For more information, see
            [Retrieve a bank account by using an ID issued by V1 Bank Accounts API](https://developer.squareup.com/docs/bank-accounts-api#retrieve-a-bank-account-by-using-an-id-issued-by-v1-bank-accounts-api).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetBankAccountByV1IdResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/bank-accounts/by-v1-id/{jsonable_encoder(v1bank_account_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetBankAccountByV1IdResponse,
                    construct_type(
                        type_=GetBankAccountByV1IdResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, bank_account_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetBankAccountResponse]:
        """
        Returns details of a [BankAccount](entity:BankAccount)
        linked to a Square account.

        Parameters
        ----------
        bank_account_id : str
            Square-issued ID of the desired `BankAccount`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetBankAccountResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/bank-accounts/{jsonable_encoder(bank_account_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetBankAccountResponse,
                    construct_type(
                        type_=GetBankAccountResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from .raw_client import RawCardsClient
from ...requests.address import AddressParams
from ...core.request_options import RequestOptions
from ...types.create_customer_card_response import CreateCustomerCardResponse
from ...types.delete_customer_card_response import DeleteCustomerCardResponse
from ...core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawCardsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CardsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCardsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawCardsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCardsClient
        """
        return self._raw_client

    def create(
        self,
        customer_id: str,
        *,
        card_nonce: str,
        billing_address: typing.Optional[AddressParams] = OMIT,
        cardholder_name: typing.Optional[str] = OMIT,
        verification_token: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateCustomerCardResponse:
        """
        Adds a card on file to an existing customer.

        As with charges, calls to `CreateCustomerCard` are idempotent. Multiple
        calls with the same card nonce return the same card record that was created
        with the provided nonce during the _first_ call.

        Parameters
        ----------
        customer_id : str
            The Square ID of the customer profile the card is linked to.

        card_nonce : str
            A card nonce representing the credit card to link to the customer.

            Card nonces are generated by the Square payment form when customers enter
            their card information. For more information, see
            [Walkthrough: Integrate Square Payments in a Website](https://developer.squareup.com/docs/web-payments/take-card-payment).

            __NOTE:__ Card nonces generated by digital wallets (such as Apple Pay)
            cannot be used to create a customer card.

        billing_address : typing.Optional[AddressParams]
            Address information for the card on file.

            __NOTE:__ If a billing address is provided in the request, the
            `CreateCustomerCardRequest.billing_address.postal_code` must match
            the postal code encoded in the card nonce.

        cardholder_name : typing.Optional[str]
            The full name printed on the credit card.

        verification_token : typing.Optional[str]
            An identifying token generated by [Payments.verifyBuyer()](https://developer.squareup.com/reference/sdks/web/payments/objects/Payments#Payments.verifyBuyer).
            Verification tokens encapsulate customer device information and 3-D Secure
            challenge results to indicate that Square has verified the buyer identity.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateCustomerCardResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.customers.cards.create(
            customer_id="customer_id",
            card_nonce="YOUR_CARD_NONCE",
            billing_address={
                "address_line1": "500 Electric Ave",
                "address_line2": "Suite 600",
                "locality": "New York",
                "administrative_district_level1": "NY",
                "postal_code": "10003",
                "country": "US",
            },
            cardholder_name="Amelia Earhart",
        )
        """
        response = self._raw_client.create(
            customer_id,
            card_nonce=card_nonce,
            billing_address=billing_address,
            cardholder_name=cardholder_name,
            verification_token=verification_token,
            request_options=request_options,
        )
        return response.data

    def delete(
        self, customer_id: str, card_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteCustomerCardResponse:
        """
        Removes a card on file from a customer.

        Parameters
        ----------
        customer_id : str
            The ID of the customer that the card on file belongs to.

        card_id : str
            The ID of the card on file to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteCustomerCardResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.customers.cards.delete(
            customer_id="customer_id",
            card_id="card_id",
        )
        """
        response = self._raw_client.delete(customer_id, card_id, request_options=request_options)
        return response.data


class AsyncCardsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCardsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawCardsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCardsClient
        """
        return self._raw_client

    async def create(
        self,
        customer_id: str,
        *,
        card_nonce: str,
        billing_address: typing.Optional[AddressParams] = OMIT,
        cardholder_name: typing.Optional[str] = OMIT,
        verification_token: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateCustomerCardResponse:
        """
        Adds a card on file to an existing customer.

        As with charges, calls to `CreateCustomerCard` are idempotent. Multiple
        calls with the same card nonce return the same card record that was created
        with the provided nonce during the _first_ call.

        Parameters
        ----------
        customer_id : str
            The Square ID of the customer profile the card is linked to.

        card_nonce : str
            A card nonce representing the credit card to link to the customer.

            Card nonces are generated by the Square payment form when customers enter
            their card information. For more information, see
            [Walkthrough: Integrate Square Payments in a Website](https://developer.squareup.com/docs/web-payments/take-card-payment).

            __NOTE:__ Card nonces generated by digital wallets (such as Apple Pay)
            cannot be used to create a customer card.

        billing_address : typing.Optional[AddressParams]
            Address information for the card on file.

            __NOTE:__ If a billing address is provided in the request, the
            `CreateCustomerCardRequest.billing_address.postal_code` must match
            the postal code encoded in the card nonce.

        cardholder_name : typing.Optional[str]
            The full name printed on the credit card.

        verification_token : typing.Optional[str]
            An identifying token generated by [Payments.verifyBuyer()](https://developer.squareup.com/reference/sdks/web/payments/objects/Payments#Payments.verifyBuyer).
            Verification tokens encapsulate customer device information and 3-D Secure
            challenge results to indicate that Square has verified the buyer identity.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateCustomerCardResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.customers.cards.create(
                customer_id="customer_id",
                card_nonce="YOUR_CARD_NONCE",
                billing_address={
                    "address_line1": "500 Electric Ave",
                    "address_line2": "Suite 600",
                    "locality": "New York",
                    "administrative_district_level1": "NY",
                    "postal_code": "10003",
                    "country": "US",
                },
                cardholder_name="Amelia Earhart",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.create(
            customer_id,
            card_nonce=card_nonce,
            billing_address=billing_address,
            cardholder_name=cardholder_name,
            verification_token=verification_token,
            request_options=request_options,
        )
        return response.data

    async def delete(
        self, customer_id: str, card_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteCustomerCardResponse:
        """
        Removes a card on file from a customer.

        Parameters
        ----------
        customer_id : str
            The ID of the customer that the card on file belongs to.

        card_id : str
            The ID of the card on file to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteCustomerCardResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.customers.cards.delete(
                customer_id="customer_id",
                card_id="card_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.delete(customer_id, card_id, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

import typing

CustomAttributeDefinitionVisibility = typing.Union[
    typing.Literal["VISIBILITY_HIDDEN", "VISIBILITY_READ_ONLY", "VISIBILITY_READ_WRITE_VALUES"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .location import LocationParams


class GetLocationResponseParams(typing_extensions.TypedDict):
    """
    Defines the fields that the [RetrieveLocation](api-endpoint:Locations-RetrieveLocation)
    endpoint returns in a response.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Information about errors encountered during the request.
    """

    location: typing_extensions.NotRequired[LocationParams]
    """
    The requested location.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

BookingStatus = typing.Union[
    typing.Literal["PENDING", "CANCELLED_BY_CUSTOMER", "CANCELLED_BY_SELLER", "DECLINED", "ACCEPTED", "NO_SHOW"],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions


class InvoiceAttachmentParams(typing_extensions.TypedDict):
    """
    Represents a file attached to an [invoice](entity:Invoice).
    """

    id: typing_extensions.NotRequired[str]
    """
    The Square-assigned ID of the attachment.
    """

    filename: typing_extensions.NotRequired[str]
    """
    The file name of the attachment, which is displayed on the invoice.
    """

    description: typing_extensions.NotRequired[str]
    """
    The description of the attachment, which is displayed on the invoice.
    This field maps to the seller-defined **Message** field.
    """

    filesize: typing_extensions.NotRequired[int]
    """
    The file size of the attachment in bytes.
    """

    hash: typing_extensions.NotRequired[str]
    """
    The MD5 hash that was generated from the file contents.
    """

    mime_type: typing_extensions.NotRequired[str]
    """
    The mime type of the attachment.
    The following mime types are supported: 
    image/gif, image/jpeg, image/png, image/tiff, image/bmp, application/pdf.
    """

    uploaded_at: typing_extensions.NotRequired[str]
    """
    The timestamp when the attachment was uploaded, in RFC 3339 format.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .dispute_evidence_file import DisputeEvidenceFileParams
from ..types.dispute_evidence_type import DisputeEvidenceType


class DisputeEvidenceParams(typing_extensions.TypedDict):
    evidence_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The Square-generated ID of the evidence.
    """

    id: typing_extensions.NotRequired[str]
    """
    The Square-generated ID of the evidence.
    """

    dispute_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the dispute the evidence is associated with.
    """

    evidence_file: typing_extensions.NotRequired[DisputeEvidenceFileParams]
    """
    Image, PDF, TXT
    """

    evidence_text: typing_extensions.NotRequired[typing.Optional[str]]
    """
    Raw text
    """

    uploaded_at: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The time when the evidence was uploaded, in RFC 3339 format.
    """

    evidence_type: typing_extensions.NotRequired[DisputeEvidenceType]
    """
    The type of the evidence.
    See [DisputeEvidenceType](#type-disputeevidencetype) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from .money import MoneyParams


class QuickPayParams(typing_extensions.TypedDict):
    """
    Describes an ad hoc item and price to generate a quick pay checkout link.
    For more information,
    see [Quick Pay Checkout](https://developer.squareup.com/docs/checkout-api/quick-pay-checkout).
    """

    name: str
    """
    The ad hoc item name. In the resulting `Order`, this name appears as the line item name.
    """

    price_money: MoneyParams
    """
    The price of the item.
    """

    location_id: str
    """
    The ID of the business location the checkout is associated with.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

ActivityType = typing.Union[
    typing.Literal[
        "ADJUSTMENT",
        "APP_FEE_REFUND",
        "APP_FEE_REVENUE",
        "AUTOMATIC_SAVINGS",
        "AUTOMATIC_SAVINGS_REVERSED",
        "CHARGE",
        "DEPOSIT_FEE",
        "DEPOSIT_FEE_REVERSED",
        "DISPUTE",
        "ESCHEATMENT",
        "FEE",
        "FREE_PROCESSING",
        "HOLD_ADJUSTMENT",
        "INITIAL_BALANCE_CHANGE",
        "MONEY_TRANSFER",
        "MONEY_TRANSFER_REVERSAL",
        "OPEN_DISPUTE",
        "OTHER",
        "OTHER_ADJUSTMENT",
        "PAID_SERVICE_FEE",
        "PAID_SERVICE_FEE_REFUND",
        "REDEMPTION_CODE",
        "REFUND",
        "RELEASE_ADJUSTMENT",
        "RESERVE_HOLD",
        "RESERVE_RELEASE",
        "RETURNED_PAYOUT",
        "SQUARE_CAPITAL_PAYMENT",
        "SQUARE_CAPITAL_REVERSED_PAYMENT",
        "SUBSCRIPTION_FEE",
        "SUBSCRIPTION_FEE_PAID_REFUND",
        "SUBSCRIPTION_FEE_REFUND",
        "TAX_ON_FEE",
        "THIRD_PARTY_FEE",
        "THIRD_PARTY_FEE_REFUND",
        "PAYOUT",
        "AUTOMATIC_BITCOIN_CONVERSIONS",
        "AUTOMATIC_BITCOIN_CONVERSIONS_REVERSED",
        "CREDIT_CARD_REPAYMENT",
        "CREDIT_CARD_REPAYMENT_REVERSED",
        "LOCAL_OFFERS_CASHBACK",
        "LOCAL_OFFERS_FEE",
        "PERCENTAGE_PROCESSING_ENROLLMENT",
        "PERCENTAGE_PROCESSING_DEACTIVATION",
        "PERCENTAGE_PROCESSING_REPAYMENT",
        "PERCENTAGE_PROCESSING_REPAYMENT_REVERSED",
        "PROCESSING_FEE",
        "PROCESSING_FEE_REFUND",
        "UNDO_PROCESSING_FEE_REFUND",
        "GIFT_CARD_LOAD_FEE",
        "GIFT_CARD_LOAD_FEE_REFUND",
        "UNDO_GIFT_CARD_LOAD_FEE_REFUND",
        "BALANCE_FOLDERS_TRANSFER",
        "BALANCE_FOLDERS_TRANSFER_REVERSED",
        "GIFT_CARD_POOL_TRANSFER",
        "GIFT_CARD_POOL_TRANSFER_REVERSED",
        "SQUARE_PAYROLL_TRANSFER",
        "SQUARE_PAYROLL_TRANSFER_REVERSED",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .team_member_booking_profile import TeamMemberBookingProfileParams
from .error import ErrorParams


class ListTeamMemberBookingProfilesResponseParams(typing_extensions.TypedDict):
    team_member_booking_profiles: typing_extensions.NotRequired[typing.Sequence[TeamMemberBookingProfileParams]]
    """
    The list of team member booking profiles. The results are returned in the ascending order of the time
    when the team member booking profiles were last updated. Multiple booking profiles updated at the same time
    are further sorted in the ascending order of their IDs.
    """

    cursor: typing_extensions.NotRequired[str]
    """
    The pagination cursor to be used in the subsequent request to get the next page of the results. Stop retrieving the next page of the results when the cursor is not set.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from ..core.unchecked_base_model import UncheckedBaseModel
from .catalog_category import CatalogCategory
from .catalog_object_category import CatalogObjectCategory
import pydantic
import typing
from .subscription_phase import SubscriptionPhase
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.pydantic_utilities import update_forward_refs


class CatalogSubscriptionPlan(UncheckedBaseModel):
    """
    Describes a subscription plan. A subscription plan represents what you want to sell in a subscription model, and includes references to each of the associated subscription plan variations.
    For more information, see [Subscription Plans and Variations](https://developer.squareup.com/docs/subscriptions-api/plans-and-variations).
    """

    name: str = pydantic.Field()
    """
    The name of the plan.
    """

    phases: typing.Optional[typing.List[SubscriptionPhase]] = pydantic.Field(default=None)
    """
    A list of SubscriptionPhase containing the [SubscriptionPhase](entity:SubscriptionPhase) for this plan.
    This field it required. Not including this field will throw a REQUIRED_FIELD_MISSING error
    """

    subscription_plan_variations: typing.Optional[typing.List["CatalogObject"]] = pydantic.Field(default=None)
    """
    The list of subscription plan variations available for this product
    """

    eligible_item_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The list of IDs of `CatalogItems` that are eligible for subscription by this SubscriptionPlan's variations.
    """

    eligible_category_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The list of IDs of `CatalogCategory` that are eligible for subscription by this SubscriptionPlan's variations.
    """

    all_items: typing.Optional[bool] = pydantic.Field(default=None)
    """
    If true, all items in the merchant's catalog are subscribable by this SubscriptionPlan.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


from .catalog_item import CatalogItem  # noqa: E402
from .catalog_item_option import CatalogItemOption  # noqa: E402
from .catalog_modifier_list import CatalogModifierList  # noqa: E402
from .catalog_object_item import CatalogObjectItem  # noqa: E402
from .catalog_object_item_option import CatalogObjectItemOption  # noqa: E402
from .catalog_object_modifier_list import CatalogObjectModifierList  # noqa: E402
from .catalog_object_subscription_plan import CatalogObjectSubscriptionPlan  # noqa: E402
from .catalog_object import CatalogObject  # noqa: E402

update_forward_refs(CatalogSubscriptionPlan)



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
from .money import Money
import pydantic
import typing
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CashPaymentDetails(UncheckedBaseModel):
    """
    Stores details about a cash payment. Contains only non-confidential information. For more information, see
    [Take Cash Payments](https://developer.squareup.com/docs/payments-api/take-payments/cash-payments).
    """

    buyer_supplied_money: Money = pydantic.Field()
    """
    The amount and currency of the money supplied by the buyer.
    """

    change_back_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The amount of change due back to the buyer. 
    This read-only field is calculated
    from the `amount_money` and `buyer_supplied_money` fields.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import pydantic
import typing
from .money import Money
from .refund_status import RefundStatus
from .additional_recipient import AdditionalRecipient
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Refund(UncheckedBaseModel):
    """
    Represents a refund processed for a Square transaction.
    """

    id: str = pydantic.Field()
    """
    The refund's unique ID.
    """

    location_id: str = pydantic.Field()
    """
    The ID of the refund's associated location.
    """

    transaction_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the transaction that the refunded tender is part of.
    """

    tender_id: str = pydantic.Field()
    """
    The ID of the refunded tender.
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp for when the refund was created, in RFC 3339 format.
    """

    reason: str = pydantic.Field()
    """
    The reason for the refund being issued.
    """

    amount_money: Money = pydantic.Field()
    """
    The amount of money refunded to the buyer.
    """

    status: RefundStatus = pydantic.Field()
    """
    The current status of the refund (`PENDING`, `APPROVED`, `REJECTED`,
    or `FAILED`).
    See [RefundStatus](#type-refundstatus) for possible values
    """

    processing_fee_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The amount of Square processing fee money refunded to the *merchant*.
    """

    additional_recipients: typing.Optional[typing.List[AdditionalRecipient]] = pydantic.Field(default=None)
    """
    Additional recipients (other than the merchant) receiving a portion of this refund.
    For example, fees assessed on a refund of a purchase by a third party integration.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.v1tender_type import V1TenderType
import typing
from ..types.v1tender_card_brand import V1TenderCardBrand
from ..types.v1tender_entry_method import V1TenderEntryMethod
from .v1money import V1MoneyParams


class V1TenderParams(typing_extensions.TypedDict):
    """
    A tender represents a discrete monetary exchange. Square represents this
    exchange as a money object with a specific currency and amount, where the
    amount is given in the smallest denomination of the given currency.

    Square POS can accept more than one form of tender for a single payment (such
    as by splitting a bill between a credit card and a gift card). The `tender`
    field of the Payment object lists all forms of tender used for the payment.

    Split tender payments behave slightly differently from single tender payments:

    The receipt_url for a split tender corresponds only to the first tender listed
    in the tender field. To get the receipt URLs for the remaining tenders, use
    the receipt_url fields of the corresponding Tender objects.

    *A note on gift cards**: when a customer purchases a Square gift card from a
    merchant, the merchant receives the full amount of the gift card in the
    associated payment.

    When that gift card is used as a tender, the balance of the gift card is
    reduced and the merchant receives no funds. A `Tender` object with a type of
    `SQUARE_GIFT_CARD` indicates a gift card was used for some or all of the
    associated payment.
    """

    id: typing_extensions.NotRequired[str]
    """
    The tender's unique ID.
    """

    type: typing_extensions.NotRequired[V1TenderType]
    """
    The type of tender.
    See [V1TenderType](#type-v1tendertype) for possible values
    """

    name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A human-readable description of the tender.
    """

    employee_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the employee that processed the tender.
    """

    receipt_url: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The URL of the receipt for the tender.
    """

    card_brand: typing_extensions.NotRequired[V1TenderCardBrand]
    """
    The brand of credit card provided.
    See [V1TenderCardBrand](#type-v1tendercardbrand) for possible values
    """

    pan_suffix: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The last four digits of the provided credit card's account number.
    """

    entry_method: typing_extensions.NotRequired[V1TenderEntryMethod]
    """
    The tender's unique ID.
    See [V1TenderEntryMethod](#type-v1tenderentrymethod) for possible values
    """

    payment_note: typing_extensions.NotRequired[typing.Optional[str]]
    """
    Notes entered by the merchant about the tender at the time of payment, if any. Typically only present for tender with the type: OTHER.
    """

    total_money: typing_extensions.NotRequired[V1MoneyParams]
    """
    The total amount of money provided in this form of tender.
    """

    tendered_money: typing_extensions.NotRequired[V1MoneyParams]
    """
    The amount of total_money applied to the payment.
    """

    tendered_at: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The time when the tender was created, in ISO 8601 format.
    """

    settled_at: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The time when the tender was settled, in ISO 8601 format.
    """

    change_back_money: typing_extensions.NotRequired[V1MoneyParams]
    """
    The amount of total_money returned to the buyer as change.
    """

    refunded_money: typing_extensions.NotRequired[V1MoneyParams]
    """
    The total of all refunds applied to this tender. This amount is always negative or zero.
    """

    is_exchange: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Indicates whether or not the tender is associated with an exchange. If is_exchange is true, the tender represents the value of goods returned in an exchange not the actual money paid. The exchange value reduces the tender amounts needed to pay for items purchased in the exchange.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.tender_bank_account_details_status import TenderBankAccountDetailsStatus


class TenderBankAccountDetailsParams(typing_extensions.TypedDict):
    """
    Represents the details of a tender with `type` `BANK_ACCOUNT`.

    See [BankAccountPaymentDetails](entity:BankAccountPaymentDetails)
    for more exposed details of a bank account payment.
    """

    status: typing_extensions.NotRequired[TenderBankAccountDetailsStatus]
    """
    The bank account payment's current state.
    
    See [TenderBankAccountPaymentDetailsStatus](entity:TenderBankAccountDetailsStatus) for possible values.
    See [TenderBankAccountDetailsStatus](#type-tenderbankaccountdetailsstatus) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

import typing

V1TenderCardBrand = typing.Union[
    typing.Literal[
        "OTHER_BRAND",
        "VISA",
        "MASTER_CARD",
        "AMERICAN_EXPRESS",
        "DISCOVER",
        "DISCOVER_DINERS",
        "JCB",
        "CHINA_UNIONPAY",
        "SQUARE_GIFT_CARD",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..types.catalog_custom_attribute_definition_type import CatalogCustomAttributeDefinitionType
import typing_extensions
import typing
from .source_application import SourceApplicationParams
from ..types.catalog_object_type import CatalogObjectType
from ..types.catalog_custom_attribute_definition_seller_visibility import (
    CatalogCustomAttributeDefinitionSellerVisibility,
)
from ..types.catalog_custom_attribute_definition_app_visibility import CatalogCustomAttributeDefinitionAppVisibility
from .catalog_custom_attribute_definition_string_config import CatalogCustomAttributeDefinitionStringConfigParams
from .catalog_custom_attribute_definition_number_config import CatalogCustomAttributeDefinitionNumberConfigParams
from .catalog_custom_attribute_definition_selection_config import CatalogCustomAttributeDefinitionSelectionConfigParams


class CatalogCustomAttributeDefinitionParams(typing_extensions.TypedDict):
    """
    Contains information defining a custom attribute. Custom attributes are
    intended to store additional information about a catalog object or to associate a
    catalog object with an entity in another system. Do not use custom attributes
    to store any sensitive information (personally identifiable information, card details, etc.).
    [Read more about custom attributes](https://developer.squareup.com/docs/catalog-api/add-custom-attributes)
    """

    type: CatalogCustomAttributeDefinitionType
    """
    The type of this custom attribute. Cannot be modified after creation.
    Required.
    See [CatalogCustomAttributeDefinitionType](#type-catalogcustomattributedefinitiontype) for possible values
    """

    name: str
    """
     The name of this definition for API and seller-facing UI purposes.
    The name must be unique within the (merchant, application) pair. Required.
    May not be empty and may not exceed 255 characters. Can be modified after creation.
    """

    description: typing_extensions.NotRequired[typing.Optional[str]]
    """
    Seller-oriented description of the meaning of this Custom Attribute,
    any constraints that the seller should observe, etc. May be displayed as a tooltip in Square UIs.
    """

    source_application: typing_extensions.NotRequired[SourceApplicationParams]
    """
    __Read only.__ Contains information about the application that
    created this custom attribute definition.
    """

    allowed_object_types: typing.Sequence[CatalogObjectType]
    """
    The set of `CatalogObject` types that this custom atttribute may be applied to.
    Currently, only `ITEM`, `ITEM_VARIATION`, `MODIFIER`, `MODIFIER_LIST`, and `CATEGORY` are allowed. At least one type must be included.
    See [CatalogObjectType](#type-catalogobjecttype) for possible values
    """

    seller_visibility: typing_extensions.NotRequired[CatalogCustomAttributeDefinitionSellerVisibility]
    """
    The visibility of a custom attribute in seller-facing UIs (including Square Point
    of Sale applications and Square Dashboard). May be modified.
    See [CatalogCustomAttributeDefinitionSellerVisibility](#type-catalogcustomattributedefinitionsellervisibility) for possible values
    """

    app_visibility: typing_extensions.NotRequired[CatalogCustomAttributeDefinitionAppVisibility]
    """
    The visibility of a custom attribute to applications other than the application
    that created the attribute.
    See [CatalogCustomAttributeDefinitionAppVisibility](#type-catalogcustomattributedefinitionappvisibility) for possible values
    """

    string_config: typing_extensions.NotRequired[CatalogCustomAttributeDefinitionStringConfigParams]
    """
    Optionally, populated when `type` = `STRING`, unset otherwise.
    """

    number_config: typing_extensions.NotRequired[CatalogCustomAttributeDefinitionNumberConfigParams]
    """
    Optionally, populated when `type` = `NUMBER`, unset otherwise.
    """

    selection_config: typing_extensions.NotRequired[CatalogCustomAttributeDefinitionSelectionConfigParams]
    """
    Populated when `type` is set to `SELECTION`, unset otherwise.
    """

    custom_attribute_usage_count: typing_extensions.NotRequired[int]
    """
    The number of custom attributes that reference this
    custom attribute definition. Set by the server in response to a ListCatalog
    request with `include_counts` set to `true`.  If the actual count is greater
    than 100, `custom_attribute_usage_count` will be set to `100`.
    """

    key: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The name of the desired custom attribute key that can be used to access
    the custom attribute value on catalog objects. Cannot be modified after the
    custom attribute definition has been created.
    Must be between 1 and 60 characters, and may only contain the characters `[a-zA-Z0-9_-]`.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .payout import PayoutParams
import typing
from .error import ErrorParams


class GetPayoutResponseParams(typing_extensions.TypedDict):
    payout: typing_extensions.NotRequired[PayoutParams]
    """
    The requested payout.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Information about errors encountered during the request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from .money import MoneyParams
import typing_extensions
import typing


class GiftCardActivityUnlinkedActivityRefundParams(typing_extensions.TypedDict):
    """
    Represents details about an `UNLINKED_ACTIVITY_REFUND` [gift card activity type](entity:GiftCardActivityType).
    """

    amount_money: MoneyParams
    """
    The amount added to the gift card for the refund. This value is a positive integer.
    """

    reference_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A client-specified ID that associates the gift card activity with an entity in another system.
    """

    payment_id: typing_extensions.NotRequired[str]
    """
    The ID of the refunded payment. This field is not used starting in Square version 2022-06-16.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .terminal_action_query_filter import TerminalActionQueryFilterParams
from .terminal_action_query_sort import TerminalActionQuerySortParams


class TerminalActionQueryParams(typing_extensions.TypedDict):
    filter: typing_extensions.NotRequired[TerminalActionQueryFilterParams]
    """
    Options for filtering returned `TerminalAction`s
    """

    sort: typing_extensions.NotRequired[TerminalActionQuerySortParams]
    """
    Option for sorting returned `TerminalAction` objects.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from ..types.fulfillment_type import FulfillmentType
from ..types.fulfillment_state import FulfillmentState


class SearchOrdersFulfillmentFilterParams(typing_extensions.TypedDict):
    """
    Filter based on [order fulfillment](entity:Fulfillment) information.
    """

    fulfillment_types: typing_extensions.NotRequired[typing.Optional[typing.Sequence[FulfillmentType]]]
    """
    A list of [fulfillment types](entity:FulfillmentType) to filter
    for. The list returns orders if any of its fulfillments match any of the fulfillment types
    listed in this field.
    See [FulfillmentType](#type-fulfillmenttype) for possible values
    """

    fulfillment_states: typing_extensions.NotRequired[typing.Optional[typing.Sequence[FulfillmentState]]]
    """
    A list of [fulfillment states](entity:FulfillmentState) to filter
    for. The list returns orders if any of its fulfillments match any of the
    fulfillment states listed in this field.
    See [FulfillmentState](#type-fulfillmentstate) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .team_member import TeamMemberParams
from .error import ErrorParams


class SearchTeamMembersResponseParams(typing_extensions.TypedDict):
    """
    Represents a response from a search request containing a filtered list of `TeamMember` objects.
    """

    team_members: typing_extensions.NotRequired[typing.Sequence[TeamMemberParams]]
    """
    The filtered list of `TeamMember` objects.
    """

    cursor: typing_extensions.NotRequired[str]
    """
    The opaque cursor for fetching the next page. For more information, see
    [pagination](https://developer.squareup.com/docs/working-with-apis/pagination).
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    The errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing
import typing_extensions


class InvoiceFilterParams(typing_extensions.TypedDict):
    """
    Describes query filters to apply.
    """

    location_ids: typing.Sequence[str]
    """
    Limits the search to the specified locations. A location is required. 
    In the current implementation, only one location can be specified.
    """

    customer_ids: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    Limits the search to the specified customers, within the specified locations. 
    Specifying a customer is optional. In the current implementation, 
    a maximum of one customer can be specified.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class CatalogQueryPrefixParams(typing_extensions.TypedDict):
    """
    The query filter to return the search result whose named attribute values are prefixed by the specified attribute value.
    """

    attribute_name: str
    """
    The name of the attribute to be searched.
    """

    attribute_prefix: str
    """
    The desired prefix of the search attribute value.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import pydantic
import typing
from .dispute_evidence_type import DisputeEvidenceType
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CreateDisputeEvidenceFileRequest(UncheckedBaseModel):
    """
    Defines the parameters for a `CreateDisputeEvidenceFile` request.
    """

    idempotency_key: str = pydantic.Field()
    """
    A unique key identifying the request. For more information, see [Idempotency](https://developer.squareup.com/docs/working-with-apis/idempotency).
    """

    evidence_type: typing.Optional[DisputeEvidenceType] = pydantic.Field(default=None)
    """
    The type of evidence you are uploading.
    See [DisputeEvidenceType](#type-disputeevidencetype) for possible values
    """

    content_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The MIME type of the uploaded file.
    The type can be image/heic, image/heif, image/jpeg, application/pdf, image/png, or image/tiff.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Break(UncheckedBaseModel):
    """
    A record of a team member's break on a [timecard](entity:Timecard).
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The UUID for this object.
    """

    start_at: str = pydantic.Field()
    """
    RFC 3339; follows the same timezone information as the [timecard](entity:Timecard). Precision up to
    the minute is respected; seconds are truncated.
    """

    end_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    RFC 3339; follows the same timezone information as the [timecard](entity:Timecard). Precision up to
    the minute is respected; seconds are truncated.
    """

    break_type_id: str = pydantic.Field()
    """
    The [BreakType](entity:BreakType) that this break was templated on.
    """

    name: str = pydantic.Field()
    """
    A human-readable name.
    """

    expected_duration: str = pydantic.Field()
    """
    Format: RFC-3339 P[n]Y[n]M[n]DT[n]H[n]M[n]S. The expected length of
    the break.
    
    Example for break expected duration of 15 minutes: PT15M
    """

    is_paid: bool = pydantic.Field()
    """
    Whether this break counts towards time worked for compensation
    purposes.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from .custom_attribute import CustomAttributeParams
import typing_extensions
import typing


class BulkUpsertLocationCustomAttributesRequestLocationCustomAttributeUpsertRequestParams(typing_extensions.TypedDict):
    """
    Represents an individual upsert request in a [BulkUpsertLocationCustomAttributes](api-endpoint:LocationCustomAttributes-BulkUpsertLocationCustomAttributes)
    request. An individual request contains a location ID, the custom attribute to create or update,
    and an optional idempotency key.
    """

    location_id: str
    """
    The ID of the target [location](entity:Location).
    """

    custom_attribute: CustomAttributeParams
    """
    The custom attribute to create or update, with following fields:
    - `key`. This key must match the `key` of a custom attribute definition in the Square seller
    account. If the requesting application is not the definition owner, you must provide the qualified key.
    - `value`. This value must conform to the `schema` specified by the definition.
    For more information, see [Supported data types](https://developer.squareup.com/docs/devtools/customattributes/overview#supported-data-types)..
    - `version`. To enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
    control, specify the current version of the custom attribute. 
    If this is not important for your application, `version` can be set to -1.
    """

    idempotency_key: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A unique identifier for this individual upsert request, used to ensure idempotency.
    For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).
    """



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..core.request_options import RequestOptions
from ..core.http_response import HttpResponse
from ..types.register_domain_response import RegisterDomainResponse
from ..core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper
from ..core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawApplePayClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def register_domain(
        self, *, domain_name: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[RegisterDomainResponse]:
        """
        Activates a domain for use with Apple Pay on the Web and Square. A validation
        is performed on this domain by Apple to ensure that it is properly set up as
        an Apple Pay enabled domain.

        This endpoint provides an easy way for platform developers to bulk activate
        Apple Pay on the Web with Square for merchants using their platform.

        Note: You will need to host a valid domain verification file on your domain to support Apple Pay.  The
        current version of this file is always available at https://app.squareup.com/digital-wallets/apple-pay/apple-developer-merchantid-domain-association,
        and should be hosted at `.well_known/apple-developer-merchantid-domain-association` on your
        domain.  This file is subject to change; we strongly recommend checking for updates regularly and avoiding
        long-lived caches that might not keep in sync with the correct file version.

        To learn more about the Web Payments SDK and how to add Apple Pay, see [Take an Apple Pay Payment](https://developer.squareup.com/docs/web-payments/apple-pay).

        Parameters
        ----------
        domain_name : str
            A domain name as described in RFC-1034 that will be registered with ApplePay.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[RegisterDomainResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/apple-pay/domains",
            method="POST",
            json={
                "domain_name": domain_name,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RegisterDomainResponse,
                    construct_type(
                        type_=RegisterDomainResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawApplePayClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def register_domain(
        self, *, domain_name: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[RegisterDomainResponse]:
        """
        Activates a domain for use with Apple Pay on the Web and Square. A validation
        is performed on this domain by Apple to ensure that it is properly set up as
        an Apple Pay enabled domain.

        This endpoint provides an easy way for platform developers to bulk activate
        Apple Pay on the Web with Square for merchants using their platform.

        Note: You will need to host a valid domain verification file on your domain to support Apple Pay.  The
        current version of this file is always available at https://app.squareup.com/digital-wallets/apple-pay/apple-developer-merchantid-domain-association,
        and should be hosted at `.well_known/apple-developer-merchantid-domain-association` on your
        domain.  This file is subject to change; we strongly recommend checking for updates regularly and avoiding
        long-lived caches that might not keep in sync with the correct file version.

        To learn more about the Web Payments SDK and how to add Apple Pay, see [Take an Apple Pay Payment](https://developer.squareup.com/docs/web-payments/apple-pay).

        Parameters
        ----------
        domain_name : str
            A domain name as described in RFC-1034 that will be registered with ApplePay.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[RegisterDomainResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/apple-pay/domains",
            method="POST",
            json={
                "domain_name": domain_name,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RegisterDomainResponse,
                    construct_type(
                        type_=RegisterDomainResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions


class CustomerSegmentParams(typing_extensions.TypedDict):
    """
    Represents a group of customer profiles that match one or more predefined filter criteria.

    Segments (also known as Smart Groups) are defined and created within the Customer Directory in the
    Square Seller Dashboard or Point of Sale.
    """

    id: typing_extensions.NotRequired[str]
    """
    A unique Square-generated ID for the segment.
    """

    name: str
    """
    The name of the segment.
    """

    created_at: typing_extensions.NotRequired[str]
    """
    The timestamp when the segment was created, in RFC 3339 format.
    """

    updated_at: typing_extensions.NotRequired[str]
    """
    The timestamp when the segment was last updated, in RFC 3339 format.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class PhaseInputParams(typing_extensions.TypedDict):
    """
    Represents the arguments used to construct a new phase.
    """

    ordinal: int
    """
    index of phase in total subscription plan
    """

    order_template_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    id of order to be used in billing
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .subscription_cadence import SubscriptionCadence
from .money import Money
from .subscription_pricing import SubscriptionPricing
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SubscriptionPhase(UncheckedBaseModel):
    """
    Describes a phase in a subscription plan variation. For more information, see [Subscription Plans and Variations](https://developer.squareup.com/docs/subscriptions-api/plans-and-variations).
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Square-assigned ID of the subscription phase. This field cannot be changed after a `SubscriptionPhase` is created.
    """

    cadence: SubscriptionCadence = pydantic.Field()
    """
    The billing cadence of the phase. For example, weekly or monthly. This field cannot be changed after a `SubscriptionPhase` is created.
    See [SubscriptionCadence](#type-subscriptioncadence) for possible values
    """

    periods: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of `cadence`s the phase lasts. If not set, the phase never ends. Only the last phase can be indefinite. This field cannot be changed after a `SubscriptionPhase` is created.
    """

    recurring_price_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The amount to bill for each `cadence`. Failure to specify this field results in a `MISSING_REQUIRED_PARAMETER` error at runtime.
    """

    ordinal: typing.Optional[int] = pydantic.Field(default=None)
    """
    The position this phase appears in the sequence of phases defined for the plan, indexed from 0. This field cannot be changed after a `SubscriptionPhase` is created.
    """

    pricing: typing.Optional[SubscriptionPricing] = pydantic.Field(default=None)
    """
    The subscription pricing.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class CategoryPathToRootNodeParams(typing_extensions.TypedDict):
    """
    A node in the path from a retrieved category to its root node.
    """

    category_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The category's ID.
    """

    category_name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The category's name.
    """



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..requests.vendor import VendorParams
from ..core.request_options import RequestOptions
from ..core.http_response import HttpResponse
from ..types.batch_create_vendors_response import BatchCreateVendorsResponse
from ..core.serialization import convert_and_respect_annotation_metadata
from ..core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..types.batch_get_vendors_response import BatchGetVendorsResponse
from ..requests.update_vendor_request import UpdateVendorRequestParams
from ..types.batch_update_vendors_response import BatchUpdateVendorsResponse
from ..types.create_vendor_response import CreateVendorResponse
from ..requests.search_vendors_request_filter import SearchVendorsRequestFilterParams
from ..requests.search_vendors_request_sort import SearchVendorsRequestSortParams
from ..types.search_vendors_response import SearchVendorsResponse
from ..types.get_vendor_response import GetVendorResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..types.update_vendor_response import UpdateVendorResponse
from ..core.client_wrapper import AsyncClientWrapper
from ..core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawVendorsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def batch_create(
        self, *, vendors: typing.Dict[str, VendorParams], request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[BatchCreateVendorsResponse]:
        """
        Creates one or more [Vendor](entity:Vendor) objects to represent suppliers to a seller.

        Parameters
        ----------
        vendors : typing.Dict[str, VendorParams]
            Specifies a set of new [Vendor](entity:Vendor) objects as represented by a collection of idempotency-key/`Vendor`-object pairs.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[BatchCreateVendorsResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/vendors/bulk-create",
            method="POST",
            json={
                "vendors": convert_and_respect_annotation_metadata(
                    object_=vendors, annotation=typing.Dict[str, VendorParams], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BatchCreateVendorsResponse,
                    construct_type(
                        type_=BatchCreateVendorsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def batch_get(
        self,
        *,
        vendor_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[BatchGetVendorsResponse]:
        """
        Retrieves one or more vendors of specified [Vendor](entity:Vendor) IDs.

        Parameters
        ----------
        vendor_ids : typing.Optional[typing.Sequence[str]]
            IDs of the [Vendor](entity:Vendor) objects to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[BatchGetVendorsResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/vendors/bulk-retrieve",
            method="POST",
            json={
                "vendor_ids": vendor_ids,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BatchGetVendorsResponse,
                    construct_type(
                        type_=BatchGetVendorsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def batch_update(
        self,
        *,
        vendors: typing.Dict[str, UpdateVendorRequestParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[BatchUpdateVendorsResponse]:
        """
        Updates one or more of existing [Vendor](entity:Vendor) objects as suppliers to a seller.

        Parameters
        ----------
        vendors : typing.Dict[str, UpdateVendorRequestParams]
            A set of [UpdateVendorRequest](entity:UpdateVendorRequest) objects encapsulating to-be-updated [Vendor](entity:Vendor)
            objects. The set is represented by  a collection of `Vendor`-ID/`UpdateVendorRequest`-object pairs.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[BatchUpdateVendorsResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/vendors/bulk-update",
            method="PUT",
            json={
                "vendors": convert_and_respect_annotation_metadata(
                    object_=vendors, annotation=typing.Dict[str, UpdateVendorRequestParams], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BatchUpdateVendorsResponse,
                    construct_type(
                        type_=BatchUpdateVendorsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        idempotency_key: str,
        vendor: typing.Optional[VendorParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateVendorResponse]:
        """
        Creates a single [Vendor](entity:Vendor) object to represent a supplier to a seller.

        Parameters
        ----------
        idempotency_key : str
            A client-supplied, universally unique identifier (UUID) to make this [CreateVendor](api-endpoint:Vendors-CreateVendor) call idempotent.

            See [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) in the
            [API Development 101](https://developer.squareup.com/docs/buildbasics) section for more
            information.

        vendor : typing.Optional[VendorParams]
            The requested [Vendor](entity:Vendor) to be created.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateVendorResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/vendors/create",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "vendor": convert_and_respect_annotation_metadata(
                    object_=vendor, annotation=VendorParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateVendorResponse,
                    construct_type(
                        type_=CreateVendorResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def search(
        self,
        *,
        filter: typing.Optional[SearchVendorsRequestFilterParams] = OMIT,
        sort: typing.Optional[SearchVendorsRequestSortParams] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SearchVendorsResponse]:
        """
        Searches for vendors using a filter against supported [Vendor](entity:Vendor) properties and a supported sorter.

        Parameters
        ----------
        filter : typing.Optional[SearchVendorsRequestFilterParams]
            Specifies a filter used to search for vendors.

        sort : typing.Optional[SearchVendorsRequestSortParams]
            Specifies a sorter used to sort the returned vendors.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this to retrieve the next set of results for the original query.

            See the [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination) guide for more information.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SearchVendorsResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/vendors/search",
            method="POST",
            json={
                "filter": convert_and_respect_annotation_metadata(
                    object_=filter, annotation=SearchVendorsRequestFilterParams, direction="write"
                ),
                "sort": convert_and_respect_annotation_metadata(
                    object_=sort, annotation=SearchVendorsRequestSortParams, direction="write"
                ),
                "cursor": cursor,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SearchVendorsResponse,
                    construct_type(
                        type_=SearchVendorsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self, vendor_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetVendorResponse]:
        """
        Retrieves the vendor of a specified [Vendor](entity:Vendor) ID.

        Parameters
        ----------
        vendor_id : str
            ID of the [Vendor](entity:Vendor) to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetVendorResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/vendors/{jsonable_encoder(vendor_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetVendorResponse,
                    construct_type(
                        type_=GetVendorResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        vendor_id: str,
        *,
        vendor: VendorParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateVendorResponse]:
        """
        Updates an existing [Vendor](entity:Vendor) object as a supplier to a seller.

        Parameters
        ----------
        vendor_id : str
            ID of the [Vendor](entity:Vendor) to retrieve.

        vendor : VendorParams
            The specified [Vendor](entity:Vendor) to be updated.

        idempotency_key : typing.Optional[str]
            A client-supplied, universally unique identifier (UUID) for the
            request.

            See [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) in the
            [API Development 101](https://developer.squareup.com/docs/buildbasics) section for more
            information.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateVendorResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/vendors/{jsonable_encoder(vendor_id)}",
            method="PUT",
            json={
                "idempotency_key": idempotency_key,
                "vendor": convert_and_respect_annotation_metadata(
                    object_=vendor, annotation=VendorParams, direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateVendorResponse,
                    construct_type(
                        type_=UpdateVendorResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawVendorsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def batch_create(
        self, *, vendors: typing.Dict[str, VendorParams], request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[BatchCreateVendorsResponse]:
        """
        Creates one or more [Vendor](entity:Vendor) objects to represent suppliers to a seller.

        Parameters
        ----------
        vendors : typing.Dict[str, VendorParams]
            Specifies a set of new [Vendor](entity:Vendor) objects as represented by a collection of idempotency-key/`Vendor`-object pairs.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[BatchCreateVendorsResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/vendors/bulk-create",
            method="POST",
            json={
                "vendors": convert_and_respect_annotation_metadata(
                    object_=vendors, annotation=typing.Dict[str, VendorParams], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BatchCreateVendorsResponse,
                    construct_type(
                        type_=BatchCreateVendorsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def batch_get(
        self,
        *,
        vendor_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[BatchGetVendorsResponse]:
        """
        Retrieves one or more vendors of specified [Vendor](entity:Vendor) IDs.

        Parameters
        ----------
        vendor_ids : typing.Optional[typing.Sequence[str]]
            IDs of the [Vendor](entity:Vendor) objects to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[BatchGetVendorsResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/vendors/bulk-retrieve",
            method="POST",
            json={
                "vendor_ids": vendor_ids,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BatchGetVendorsResponse,
                    construct_type(
                        type_=BatchGetVendorsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def batch_update(
        self,
        *,
        vendors: typing.Dict[str, UpdateVendorRequestParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[BatchUpdateVendorsResponse]:
        """
        Updates one or more of existing [Vendor](entity:Vendor) objects as suppliers to a seller.

        Parameters
        ----------
        vendors : typing.Dict[str, UpdateVendorRequestParams]
            A set of [UpdateVendorRequest](entity:UpdateVendorRequest) objects encapsulating to-be-updated [Vendor](entity:Vendor)
            objects. The set is represented by  a collection of `Vendor`-ID/`UpdateVendorRequest`-object pairs.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[BatchUpdateVendorsResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/vendors/bulk-update",
            method="PUT",
            json={
                "vendors": convert_and_respect_annotation_metadata(
                    object_=vendors, annotation=typing.Dict[str, UpdateVendorRequestParams], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BatchUpdateVendorsResponse,
                    construct_type(
                        type_=BatchUpdateVendorsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        idempotency_key: str,
        vendor: typing.Optional[VendorParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateVendorResponse]:
        """
        Creates a single [Vendor](entity:Vendor) object to represent a supplier to a seller.

        Parameters
        ----------
        idempotency_key : str
            A client-supplied, universally unique identifier (UUID) to make this [CreateVendor](api-endpoint:Vendors-CreateVendor) call idempotent.

            See [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) in the
            [API Development 101](https://developer.squareup.com/docs/buildbasics) section for more
            information.

        vendor : typing.Optional[VendorParams]
            The requested [Vendor](entity:Vendor) to be created.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateVendorResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/vendors/create",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "vendor": convert_and_respect_annotation_metadata(
                    object_=vendor, annotation=VendorParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateVendorResponse,
                    construct_type(
                        type_=CreateVendorResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def search(
        self,
        *,
        filter: typing.Optional[SearchVendorsRequestFilterParams] = OMIT,
        sort: typing.Optional[SearchVendorsRequestSortParams] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SearchVendorsResponse]:
        """
        Searches for vendors using a filter against supported [Vendor](entity:Vendor) properties and a supported sorter.

        Parameters
        ----------
        filter : typing.Optional[SearchVendorsRequestFilterParams]
            Specifies a filter used to search for vendors.

        sort : typing.Optional[SearchVendorsRequestSortParams]
            Specifies a sorter used to sort the returned vendors.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this to retrieve the next set of results for the original query.

            See the [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination) guide for more information.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SearchVendorsResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/vendors/search",
            method="POST",
            json={
                "filter": convert_and_respect_annotation_metadata(
                    object_=filter, annotation=SearchVendorsRequestFilterParams, direction="write"
                ),
                "sort": convert_and_respect_annotation_metadata(
                    object_=sort, annotation=SearchVendorsRequestSortParams, direction="write"
                ),
                "cursor": cursor,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SearchVendorsResponse,
                    construct_type(
                        type_=SearchVendorsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, vendor_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetVendorResponse]:
        """
        Retrieves the vendor of a specified [Vendor](entity:Vendor) ID.

        Parameters
        ----------
        vendor_id : str
            ID of the [Vendor](entity:Vendor) to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetVendorResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/vendors/{jsonable_encoder(vendor_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetVendorResponse,
                    construct_type(
                        type_=GetVendorResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        vendor_id: str,
        *,
        vendor: VendorParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateVendorResponse]:
        """
        Updates an existing [Vendor](entity:Vendor) object as a supplier to a seller.

        Parameters
        ----------
        vendor_id : str
            ID of the [Vendor](entity:Vendor) to retrieve.

        vendor : VendorParams
            The specified [Vendor](entity:Vendor) to be updated.

        idempotency_key : typing.Optional[str]
            A client-supplied, universally unique identifier (UUID) for the
            request.

            See [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) in the
            [API Development 101](https://developer.squareup.com/docs/buildbasics) section for more
            information.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateVendorResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/vendors/{jsonable_encoder(vendor_id)}",
            method="PUT",
            json={
                "idempotency_key": idempotency_key,
                "vendor": convert_and_respect_annotation_metadata(
                    object_=vendor, annotation=VendorParams, direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateVendorResponse,
                    construct_type(
                        type_=UpdateVendorResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

DestinationType = typing.Union[
    typing.Literal["BANK_ACCOUNT", "CARD", "SQUARE_BALANCE", "SQUARE_STORED_BALANCE"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class DeviceMetadataParams(typing_extensions.TypedDict):
    battery_percentage: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The Terminal’s remaining battery percentage, between 1-100.
    """

    charging_state: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The current charging state of the Terminal.
    Options: `CHARGING`, `NOT_CHARGING`
    """

    location_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the Square seller business location associated with the Terminal.
    """

    merchant_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the Square merchant account that is currently signed-in to the Terminal.
    """

    network_connection_type: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The Terminal’s current network connection type.
    Options: `WIFI`, `ETHERNET`
    """

    payment_region: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The country in which the Terminal is authorized to take payments.
    """

    serial_number: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The unique identifier assigned to the Terminal, which can be found on the lower back
    of the device.
    """

    os_version: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The current version of the Terminal’s operating system.
    """

    app_version: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The current version of the application running on the Terminal.
    """

    wifi_network_name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The name of the Wi-Fi network to which the Terminal is connected.
    """

    wifi_network_strength: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The signal strength of the Wi-FI network connection.
    Options: `POOR`, `FAIR`, `GOOD`, `EXCELLENT`
    """

    ip_address: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The IP address of the Terminal.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams


class CalculateLoyaltyPointsResponseParams(typing_extensions.TypedDict):
    """
    Represents a [CalculateLoyaltyPoints](api-endpoint:Loyalty-CalculateLoyaltyPoints) response.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    points: typing_extensions.NotRequired[int]
    """
    The number of points that the buyer can earn from the base loyalty program.
    """

    promotion_points: typing_extensions.NotRequired[int]
    """
    The number of points that the buyer can earn from a loyalty promotion. To be eligible
    to earn promotion points, the purchase must first qualify for program points. When `order_id`
    is not provided in the request, this value is always 0.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .snippet import SnippetParams


class GetSnippetResponseParams(typing_extensions.TypedDict):
    """
    Represents a `RetrieveSnippet` response. The response can include either `snippet` or `errors`.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    snippet: typing_extensions.NotRequired[SnippetParams]
    """
    The retrieved snippet.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.loyalty_reward_status import LoyaltyRewardStatus


class SearchLoyaltyRewardsRequestLoyaltyRewardQueryParams(typing_extensions.TypedDict):
    """
    The set of search requirements.
    """

    loyalty_account_id: str
    """
    The ID of the [loyalty account](entity:LoyaltyAccount) to which the loyalty reward belongs.
    """

    status: typing_extensions.NotRequired[LoyaltyRewardStatus]
    """
    The status of the loyalty reward.
    See [LoyaltyRewardStatus](#type-loyaltyrewardstatus) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class DisputeEvidenceFile(UncheckedBaseModel):
    """
    A file to be uploaded as dispute evidence.
    """

    filename: typing.Optional[str] = pydantic.Field(default=None)
    """
    The file name including the file extension. For example: "receipt.tiff".
    """

    filetype: typing.Optional[str] = pydantic.Field(default=None)
    """
    Dispute evidence files must be application/pdf, image/heic, image/heif, image/jpeg, image/png, or image/tiff formats.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .filter_value import FilterValueParams


class SegmentFilterParams(typing_extensions.TypedDict):
    """
    A query filter to search for buyer-accessible appointment segments by.
    """

    service_variation_id: str
    """
    The ID of the [CatalogItemVariation](entity:CatalogItemVariation) object representing the service booked in this segment.
    """

    team_member_id_filter: typing_extensions.NotRequired[FilterValueParams]
    """
    A query filter to search for buyer-accessible appointment segments with service-providing team members matching the specified list of team member IDs.  Supported query expressions are
    - `ANY`: return the appointment segments with team members whose IDs match any member in this list.
    - `NONE`: return the appointment segments with team members whose IDs are not in this list.
    - `ALL`: not supported.
    
    When no expression is specified, any service-providing team member is eligible to fulfill the Booking.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
from .loyalty_promotion_incentive_type import LoyaltyPromotionIncentiveType
import pydantic
import typing
from .loyalty_promotion_incentive_points_multiplier_data import LoyaltyPromotionIncentivePointsMultiplierData
from .loyalty_promotion_incentive_points_addition_data import LoyaltyPromotionIncentivePointsAdditionData
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class LoyaltyPromotionIncentive(UncheckedBaseModel):
    """
    Represents how points for a [loyalty promotion](entity:LoyaltyPromotion) are calculated,
    either by multiplying the points earned from the base program or by adding a specified number
    of points to the points earned from the base program.
    """

    type: LoyaltyPromotionIncentiveType = pydantic.Field()
    """
    The type of points incentive.
    See [LoyaltyPromotionIncentiveType](#type-loyaltypromotionincentivetype) for possible values
    """

    points_multiplier_data: typing.Optional[LoyaltyPromotionIncentivePointsMultiplierData] = pydantic.Field(
        default=None
    )
    """
    Additional data for a `POINTS_MULTIPLIER` incentive type.
    """

    points_addition_data: typing.Optional[LoyaltyPromotionIncentivePointsAdditionData] = pydantic.Field(default=None)
    """
    Additional data for a `POINTS_ADDITION` incentive type.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .money import Money
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ProcessingFee(UncheckedBaseModel):
    """
    Represents the Square processing fee.
    """

    effective_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp of when the fee takes effect, in RFC 3339 format.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The type of fee assessed or adjusted. The fee type can be `INITIAL` or `ADJUSTMENT`.
    """

    amount_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The fee amount, which might be negative, that is assessed or adjusted by Square.
    
    Positive values represent funds being assessed, while negative values represent
    funds being returned.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

ArchivedState = typing.Union[
    typing.Literal["ARCHIVED_STATE_NOT_ARCHIVED", "ARCHIVED_STATE_ARCHIVED", "ARCHIVED_STATE_ALL"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.search_vendors_request_sort_field import SearchVendorsRequestSortField
from ..types.sort_order import SortOrder


class SearchVendorsRequestSortParams(typing_extensions.TypedDict):
    """
    Defines a sorter used to sort results from [SearchVendors](api-endpoint:Vendors-SearchVendors).
    """

    field: typing_extensions.NotRequired[SearchVendorsRequestSortField]
    """
    Specifies the sort key to sort the returned vendors.
    See [Field](#type-field) for possible values
    """

    order: typing_extensions.NotRequired[SortOrder]
    """
    Specifies the sort order for the returned vendors.
    See [SortOrder](#type-sortorder) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.dispute_evidence_type import DisputeEvidenceType


class CreateDisputeEvidenceFileRequestParams(typing_extensions.TypedDict):
    """
    Defines the parameters for a `CreateDisputeEvidenceFile` request.
    """

    idempotency_key: str
    """
    A unique key identifying the request. For more information, see [Idempotency](https://developer.squareup.com/docs/working-with-apis/idempotency).
    """

    evidence_type: typing_extensions.NotRequired[DisputeEvidenceType]
    """
    The type of evidence you are uploading.
    See [DisputeEvidenceType](#type-disputeevidencetype) for possible values
    """

    content_type: typing_extensions.NotRequired[str]
    """
    The MIME type of the uploaded file.
    The type can be image/heic, image/heif, image/jpeg, application/pdf, image/png, or image/tiff.
    """



# This file was auto-generated by Fern from our API Definition.

from ...core.client_wrapper import SyncClientWrapper
from .raw_client import RawEventTypesClient
import typing
from ...core.request_options import RequestOptions
from ...types.list_webhook_event_types_response import ListWebhookEventTypesResponse
from ...core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawEventTypesClient


class EventTypesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawEventTypesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawEventTypesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawEventTypesClient
        """
        return self._raw_client

    def list(
        self, *, api_version: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> ListWebhookEventTypesResponse:
        """
        Lists all webhook event types that can be subscribed to.

        Parameters
        ----------
        api_version : typing.Optional[str]
            The API version for which to list event types. Setting this field overrides the default version used by the application.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListWebhookEventTypesResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.webhooks.event_types.list()
        """
        response = self._raw_client.list(api_version=api_version, request_options=request_options)
        return response.data


class AsyncEventTypesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawEventTypesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawEventTypesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawEventTypesClient
        """
        return self._raw_client

    async def list(
        self, *, api_version: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> ListWebhookEventTypesResponse:
        """
        Lists all webhook event types that can be subscribed to.

        Parameters
        ----------
        api_version : typing.Optional[str]
            The API version for which to list event types. Setting this field overrides the default version used by the application.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListWebhookEventTypesResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.webhooks.event_types.list()


        asyncio.run(main())
        """
        response = await self._raw_client.list(api_version=api_version, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.scheduled_shift_sort_field import ScheduledShiftSortField
from ..types.sort_order import SortOrder


class ScheduledShiftSortParams(typing_extensions.TypedDict):
    """
    Defines sort criteria for a [SearchScheduledShifts](api-endpoint:Labor-SearchScheduledShifts)
    request.
    """

    field: typing_extensions.NotRequired[ScheduledShiftSortField]
    """
    The field to sort on. The default value is `START_AT`.
    See [ScheduledShiftSortField](#type-scheduledshiftsortfield) for possible values
    """

    order: typing_extensions.NotRequired[SortOrder]
    """
    The order in which results are returned. The default value is `ASC`.
    See [SortOrder](#type-sortorder) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class LocationBookingProfile(UncheckedBaseModel):
    """
    The booking profile of a seller's location, including the location's ID and whether the location is enabled for online booking.
    """

    location_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the [location](entity:Location).
    """

    booking_site_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    Url for the online booking site for this location.
    """

    online_booking_enabled: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates whether the location is enabled for online booking.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .customer_text_filter import CustomerTextFilter
import pydantic
from .filter_value import FilterValue
from .time_range import TimeRange
from .float_number_range import FloatNumberRange
from .customer_address_filter import CustomerAddressFilter
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CustomerCustomAttributeFilterValue(UncheckedBaseModel):
    """
    A type-specific filter used in a [custom attribute filter](entity:CustomerCustomAttributeFilter) to search based on the value
    of a customer-related [custom attribute](entity:CustomAttribute).
    """

    email: typing.Optional[CustomerTextFilter] = pydantic.Field(default=None)
    """
    A filter for a query based on the value of an `Email`-type custom attribute. This filter is case-insensitive and can
    include `exact` or `fuzzy`, but not both.
    
    For an `exact` match, provide the complete email address.
    
    For a `fuzzy` match, provide a query expression containing one or more query tokens to match against the email address. Square removes
    any punctuation (including periods (.), underscores (_), and the @ symbol) and tokenizes the email addresses on spaces. A match is found
    if a tokenized email address contains all the tokens in the search query, irrespective of the token order. For example, `Steven gmail`
    matches steven.jones@gmail.com and mygmail@stevensbakery.com.
    """

    phone: typing.Optional[CustomerTextFilter] = pydantic.Field(default=None)
    """
    A filter for a query based on the value of a `PhoneNumber`-type custom attribute. This filter is case-insensitive and
    can include `exact` or `fuzzy`, but not both.
    
    For an `exact` match, provide the complete phone number. This is always an E.164-compliant phone number that starts
    with the + sign followed by the country code and subscriber number. For example, the format for a US phone number is +12061112222.
    
    For a `fuzzy` match, provide a query expression containing one or more query tokens to match against the phone number.
    Square removes any punctuation and tokenizes the expression on spaces. A match is found if a tokenized phone number contains
    all the tokens in the search query, irrespective of the token order. For example, `415 123 45` is tokenized to `415`, `123`, and `45`,
    which matches +14151234567 and +12345674158, but does not match +1234156780. Similarly, the expression `415` matches
    +14151234567, +12345674158, and +1234156780.
    """

    text: typing.Optional[CustomerTextFilter] = pydantic.Field(default=None)
    """
    A filter for a query based on the value of a `String`-type custom attribute. This filter is case-insensitive and 
    can include `exact` or `fuzzy`, but not both.
    
    For an `exact` match, provide the complete string.
    
    For a `fuzzy` match, provide a query expression containing one or more query tokens in any order that contain complete words
    to match against the string. Square tokenizes the expression using a grammar-based tokenizer. For example, the expressions `quick brown`,
    `brown quick`, and `quick fox` match "The quick brown fox jumps over the lazy dog". However, `quick foxes` and `qui` do not match.
    """

    selection: typing.Optional[FilterValue] = pydantic.Field(default=None)
    """
    A filter for a query based on the display name for a `Selection`-type custom attribute value. This filter is case-sensitive
    and can contain `any`, `all`, or both. The `none` condition is not supported.
    
    Provide the display name of each item that you want to search for. To find the display names for the selection, use the 
    [Customer Custom Attributes API](api:CustomerCustomAttributes) to retrieve the corresponding custom attribute definition
    and then check the `schema.items.names` field. For more information, see
    [Search based on selection](https://developer.squareup.com/docs/customers-api/use-the-api/search-customers#custom-attribute-value-filter-selection).
    
    Note that when a `Selection`-type custom attribute is assigned to a customer profile, the custom attribute value is a list of one
    or more UUIDs (sourced from the `schema.items.enum` field) that map to the item names. These UUIDs are unique per seller.
    """

    date: typing.Optional[TimeRange] = pydantic.Field(default=None)
    """
    A filter for a query based on the value of a `Date`-type custom attribute.
    
    Provide a date range for this filter using `start_at`, `end_at`, or both. Range boundaries are inclusive. Dates can be specified
    in `YYYY-MM-DD` format or as RFC 3339 timestamps.
    """

    number: typing.Optional[FloatNumberRange] = pydantic.Field(default=None)
    """
    A filter for a query based on the value of a `Number`-type custom attribute, which can be an integer or a decimal with up to
    5 digits of precision.
    
    Provide a numerical range for this filter using `start_at`, `end_at`, or both. Range boundaries are inclusive. Numbers are specified
    as decimals or integers. The absolute value of range boundaries must not exceed `(2^63-1)/10^5`, or 92233720368547.
    """

    boolean: typing.Optional[bool] = pydantic.Field(default=None)
    """
    A filter for a query based on the value of a `Boolean`-type custom attribute.
    """

    address: typing.Optional[CustomerAddressFilter] = pydantic.Field(default=None)
    """
    A filter for a query based on the value of an `Address`-type custom attribute. The filter can include `postal_code`, `country`, or both.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
